<?php
class WPJAM_Admin{
	public static function get_screen_id(){
		static $screen_id;

		if(!isset($screen_id)){
			if(isset($_POST['screen_id'])){
				$screen_id	= $_POST['screen_id'];
			}elseif(isset($_POST['screen'])){
				$screen_id	= $_POST['screen'];	
			}else{
				$ajax_action	= $_REQUEST['action'] ?? '';

				if($ajax_action == 'fetch-list'){
					$screen_id	= $_GET['list_args']['screen']['id'];
				}elseif($ajax_action == 'inline-save-tax'){
					$screen_id	= 'edit-'.sanitize_key($_POST['taxonomy']);
				}elseif(in_array($ajax_action, ['get-comments', 'replyto-comment'])){
					$screen_id	= 'edit-comments';
				}else{
					$screen_id	= false;
				}
			}

			if($screen_id){
				if('-network' === substr($screen_id, -8)){
					if(!defined('WP_NETWORK_ADMIN')){
						define('WP_NETWORK_ADMIN', true);
					}
				}elseif('-user' === substr($screen_id, -5)){
					if(!defined('WP_USER_ADMIN')){
						define('WP_USER_ADMIN', true);
					}
				}
			}
		}

		return $screen_id;	
	}

	public static function add_error($message='', $type='success'){
		if($message){
			$admin_errors	= self::get_errors();

			if(is_wp_error($message)){
				$admin_errors[]	= ['message'=>$message->get_error_message(), 'type'=>'error'];
			}elseif($type){
				$admin_errors[]	= compact('message','type');
			}

			add_screen_option('admin_errors', $admin_errors);
		}
	}

	public static function get_errors(){
		return get_current_screen()->get_option('admin_errors') ?: [];
	}

	public static function display_errors(){
		$admin_errors	= self::get_errors();

		foreach($admin_errors as $error){
			echo '<div class="notice notice-'.$error['type'].' is-dismissible"><p>'.$error['message'].'</p></div>';
		}
	}

	public static function init($plugin_page){
		$GLOBALS['plugin_page']	= $plugin_page;

		do_action('wpjam_admin_init');

		if($plugin_page){
			WPJAM_Menu_Page::render(false);
		}

		$screen_id	= self::get_screen_id();
			
		if($screen_id == 'upload'){
			$GLOBALS['hook_suffix']	= $screen_id;

			set_current_screen();
		}else{
			set_current_screen($screen_id);
		}
	}

	public static function on_admin_menu(){
		do_action('wpjam_admin_init');

		WPJAM_Menu_Page::render();
	}

	public static function on_admin_init(){
		$plugin_page	= $_POST['plugin_page'] ?? null;

		self::init($plugin_page);	
	}

	public static function on_current_screen($screen=null){
		$object = WPJAM_Plugin_Page::get_current();

		if($object){
			$object->load();
		}elseif($screen){
			WPJAM_Builtin_Page::load($screen);
		}
	}

	public static function on_admin_enqueue_scripts(){
		$screen	= get_current_screen();
		
		if($screen->base == 'customize'){
			return;
		}elseif($screen->base == 'post'){
			wp_enqueue_media(['post'=>wpjam_get_admin_post_id()]);
		}else{
			wp_enqueue_media();
		}

		$ver	= get_plugin_data(WPJAM_BASIC_PLUGIN_FILE)['Version'];
		$static	= WPJAM_BASIC_PLUGIN_URL.'static';

		wp_enqueue_script('thickbox');
		wp_enqueue_style('thickbox');

		wp_enqueue_style('wpjam-style',		$static.'/style.css',	['wp-color-picker', 'editor-buttons'], $ver);
		wp_enqueue_script('wpjam-script',	$static.'/script.js',	['jquery', 'thickbox', 'wp-backbone', 'jquery-ui-sortable', 'jquery-ui-tooltip', 'jquery-ui-tabs', 'jquery-ui-draggable', 'jquery-ui-slider', 'jquery-ui-autocomplete', 'wp-color-picker'], $ver);
		wp_enqueue_script('wpjam-form',		$static.'/form.js',		['wpjam-script', 'mce-view'], $ver);

		$setting	= [
			'screen_base'	=> $screen->base,
			'screen_id'		=> $screen->id,
			'post_type'		=> $screen->post_type,
			'taxonomy'		=> $screen->taxonomy,
		];

		$params		= wpjam_array_except($_REQUEST, array_merge(wp_removable_query_args(),['page', 'tab', 'post_type', 'taxonomy', '_wp_http_referer', '_wpnonce']));
		$params		= array_filter($params, 'is_populated');

		if($GLOBALS['plugin_page']){
			$setting['plugin_page']	= $GLOBALS['plugin_page'];
			$setting['current_tab']	= $GLOBALS['current_tab'] ?? null;
			$setting['admin_url']	= $GLOBALS['current_admin_url'] ?? '';

			$query_data	= wpjam_get_plugin_page_query_data();

			if($query_data){
				$params	= wpjam_array_except($params, array_keys($query_data));

				$setting['query_data']	= array_map('sanitize_textarea_field', $query_data);
			}
		}

		$setting['params']	= $params ? array_map('sanitize_textarea_field', $params) : new stdClass();

		if(!empty($GLOBALS['wpjam_list_table'])){
			$setting['list_table']	= $screen->get_option('wpjam_list_table');
		}

		wp_localize_script('wpjam-script', 'wpjam_page_setting', $setting);
	}

	public static function on_admin_action_update(){
		// 为了实现多个页面使用通过 option 存储。这个可以放弃了，使用 AJAX + Redirect
		// 注册设置选项，选用的是：'admin_action_' . $_REQUEST['action'] hook，
		// 因为在这之前的 admin_init 检测 $plugin_page 的合法性

		$referer_origin	= parse_url(wpjam_get_referer());

		if(!empty($referer_origin['query'])){
			$referer_args	= wp_parse_args($referer_origin['query']);

			if(!empty($referer_args['page'])){
				self::init($referer_args['page']);	// 实现多个页面使用同个 option 存储。
			}
		}
	}
}
