<?php
if(!class_exists('WP_List_Table')){
	include ABSPATH.'wp-admin/includes/class-wp-list-table.php';
}

include WPJAM_BASIC_PLUGIN_DIR.'includes/class-wpjam-list-table.php';
include WPJAM_BASIC_PLUGIN_DIR.'includes/class-wpjam-builtin.php';
include WPJAM_BASIC_PLUGIN_DIR.'includes/class-wpjam-menu-page.php';
include WPJAM_BASIC_PLUGIN_DIR.'includes/class-wpjam-chart.php';
include WPJAM_BASIC_PLUGIN_DIR.'includes/class-wpjam-admin.php';

function wpjam_admin_add_error($message='', $type='success'){
	if(did_action('current_screen')){
		WPJAM_Admin::add_error($message, $type);
	}else{
		trigger_error('wpjam_admin_add_error too early');
	}
}

function wpjam_register_builtin_page_load($name, $args){
	return WPJAM_Builtin_Page_Load::pre_register($name, $args);
}

function wpjam_register_plugin_page_load($name, $args){
	return WPJAM_Plugin_Page_Load::pre_register($name, $args);
}

function wpjam_generate_query_data($query_args){
	$query_data	= [];

	foreach($query_args as $query_arg){
		$query_data[$query_arg]	= wpjam_get_data_parameter($query_arg);
	}

	return $query_data;
}

function wpjam_get_page_summary($type='page'){
	return (string)get_current_screen()->get_option($type.'_summary');
}

function wpjam_set_page_summary($summary, $type='page', $append=true){
	if($append){
		$summary	= wpjam_get_page_summary($type).$summary;
	}

	add_screen_option($type.'_summary', $summary);
}

function wpjam_set_plugin_page_summary($summary, $append=true){
	wpjam_set_page_summary($summary, 'page', $append);
}

function wpjam_set_builtin_page_summary($summary, $append=true){
	wpjam_set_page_summary($summary, 'page', $append);
}

function wpjam_get_plugin_page_setting($key='', $using_tab=false){
	if($object = WPJAM_Plugin_Page::get_current()){
		$is_tab	= $object->function == 'tab';

		if(str_ends_with($key, '_name')){
			$using_tab	= $is_tab;
			$default	= $GLOBALS['plugin_page'];
		}else{
			$using_tab	= $using_tab ? $is_tab : false;
			$default	= null;
		}

		if($using_tab){
			if($tab_object = $object->current_tab){
				return $key ? ($tab_object->$key ?: $default) : $tab_object->to_array();
			}
		}else{
			return $key ? ($object->$key ?: $default) : $object->to_array();
		}
	}

	return null;
}

function wpjam_get_plugin_page_type(){
	return wpjam_get_plugin_page_setting('function');
}

function wpjam_get_current_tab_setting($key=''){
	return wpjam_get_plugin_page_setting($key, true);
}

function wpjam_get_plugin_page_query_data(){
	$value	= wpjam_get_plugin_page_setting('query_data') ?: [];
		
	if($query_data = wpjam_get_current_tab_setting('query_data', true)){
		$value	= array_merge($value, $query_data);
	}

	return $value ?: [];
}

function wpjam_admin_tooltip($text, $tooltip){
	return '<div class="wpjam-tooltip">'.$text.'<div class="wpjam-tooltip-text">'.wpautop($tooltip).'</div></div>';
}

function wpjam_get_referer(){
	$referer	= wp_get_original_referer();
	$referer	= $referer ?: wp_get_referer();

	$removable_query_args	= array_merge(wp_removable_query_args(), ['_wp_http_referer', 'action', 'action2', '_wpnonce']);

	return remove_query_arg($removable_query_args, $referer);	
}

function wpjam_register_page_action($name, $args){
	return WPJAM_Page_Action::register($name, $args);
}

function wpjam_unregister_page_action($name){
	WPJAM_Page_Action::unregister($name);
}

function wpjam_get_nonce_action($key){
	$prefix	= $GLOBALS['plugin_page'] ?? $GLOBALS['current_screen']->id;

	return $prefix.'-'.$key;
}

function wpjam_get_page_button($name, $args=[]){
	$instance	= WPJAM_Page_Action::get($name);

	return $instance ? $instance->get_button($args) : '';
}

function wpjam_register_list_table($name, $args=[]){
	return WPJAM_List_Table_Setting::register($name, $args);
}

function wpjam_register_list_table_action($name, $args){
	return WPJAM_List_Table_Action::register($name, $args);
}

function wpjam_unregister_list_table_action($name){
	WPJAM_List_Table_Action::unregister($name);
}

function wpjam_register_list_table_column($name, $field){
	return WPJAM_List_Table_Column::pre_register($name, $field);
}

function wpjam_unregister_list_table_column($name, $field=[]){
	WPJAM_List_Table_Column::unregister_pre($name, $field);
}

function wpjam_register_plugin_page($name, $args){
	return WPJAM_Plugin_Page::register($name, $args);
}

function wpjam_register_plugin_page_tab($name, $args){
	return WPJAM_Plugin_Page::register_tab($name, $args);
}

function wpjam_get_list_table_filter_link($filters, $title, $class=''){
	return $GLOBALS['wpjam_list_table']->get_filter_link($filters, $title, $class);
}

function wpjam_get_list_table_row_action($action, $args=[]){
	return $GLOBALS['wpjam_list_table']->get_row_action($action, $args);
}

function wpjam_render_list_table_column_items($id, $items, $args=[]){
	return $GLOBALS['wpjam_list_table']->render_column_items($id, $items, $args);
}

function wpjam_call_list_table_model_method($method, ...$args){
	return $GLOBALS['wpjam_list_table']->call_model_method($method, ...$args);
}

function wpjam_register_dashboard($name, $args){
	return WPJAM_Dashboard_Setting::register($name, $args);
}

function wpjam_unregister_dashboard($name){
	WPJAM_Dashboard_Setting::unregister($name);
}

function wpjam_register_dashboard_widget($name, $args){
	return WPJAM_Dashboard_Widget::register($name, $args);
}

function wpjam_unregister_dashboard_widget($name){
	WPJAM_Dashboard_Widget::unregister($name);
}

function wpjam_get_admin_post_id(){
	if(isset($_GET['post'])){
		return (int)$_GET['post'];
	}elseif(isset($_POST['post_ID'])){
		return (int)$_POST['post_ID'];
	}else{
		return 0;
	}
}

function wpjam_line_chart($counts_array, $labels, $args=[], $type = 'Line'){
	WPJAM_Chart::line($counts_array, $labels, $args, $type);
}

function wpjam_bar_chart($counts_array, $labels, $args=[]){
	wpjam_line_chart($counts_array, $labels, $args, 'Bar');
}

function wpjam_donut_chart($counts, $args=[]){
	WPJAM_Chart::donut($counts, $args);
}

function wpjam_get_chart_parameter($key){
	return WPJAM_Chart::get_parameter($key);
}

function wpjam_get_current_screen_id(){
	if(did_action('current_screen')){
		return get_current_screen()->id;
	}elseif(wp_doing_ajax()){
		return WPJAM_Admin::get_screen_id();
	}
}

function wpjam_get_admin_menu_hook($type='action'){
	if(is_network_admin()){
		$prefix	= 'network_';
	}elseif(is_user_admin()){
		$prefix	= 'user_';
	}else{
		$prefix	= '';
	}

	if($type == 'action'){
		return $prefix.'admin_menu';
	}else{
		return 'wpjam_'.$prefix.'pages';
	}
}

add_action('plugins_loaded', function(){	// 内部的 hook 使用 优先级 9，因为内嵌的 hook 优先级要低
	wpjam_register_page_action('delete_notice', [
		'button_text'	=> '删除',
		'tag'			=> 'span',
		'class'			=> 'hidden delete-notice',
		'callback'		=> ['WPJAM_Notice', 'ajax_delete'],
		'direct'		=> true,
	]);

	if($GLOBALS['pagenow'] == 'options.php'){
		add_action('admin_action_update',	['WPJAM_Admin', 'on_admin_action_update'], 9);
	}elseif(wp_doing_ajax()){
		if(wpjam_get_current_screen_id()){
			add_action('admin_init',	['WPJAM_Admin', 'on_admin_init'], 9);

			add_action('wp_ajax_wpjam-page-action',	['WPJAM_Page_Action', 'ajax_response']);
			add_action('wp_ajax_wpjam-query', 		['WPJAM_Field_Data_Type', 'ajax_query']);
		}
	}else{
		$menu_action	= wpjam_get_admin_menu_hook('action');

		add_action($menu_action,			['WPJAM_Admin', 'on_admin_menu'], 9);
		add_action('admin_notices',			['WPJAM_Admin', 'display_errors']);
		add_action('admin_notices',			['WPJAM_Notice', 'on_admin_notices']);
		add_action('admin_enqueue_scripts', ['WPJAM_Admin', 'on_admin_enqueue_scripts'], 9);
		add_action('print_media_templates', ['WPJAM_Field',	'print_media_templates'], 9);

		add_filter('wpjam_html', 			['WPJAM_Queried_Menu', 'filter_html']);

		add_filter('set-screen-option', function($status, $option, $value){
			return isset($_GET['page']) ? $value : $status;
		}, 9, 3);
	}

	add_action('current_screen',	['WPJAM_Admin', 'on_current_screen'], 9);
});


