<?php
function wpjam_json_encode($data){
	return wp_json_encode($data, JSON_UNESCAPED_UNICODE);
}

function wpjam_json_decode($json, $assoc=true){
	$json	= wpjam_strip_control_characters($json);

	if(!$json){
		return new WP_Error('empty_json', 'JSON 内容不能为空！');
	}

	$result	= json_decode($json, $assoc);

	if(is_null($result)){
		$result	= json_decode(stripslashes($json), $assoc);

		if(is_null($result)){
			if(wpjam_doing_debug()){
				print_r(json_last_error());
				print_r(json_last_error_msg());
			}
			trigger_error('json_decode_error '. json_last_error_msg()."\n".var_export($json,true));
			return new WP_Error('json_decode_error', json_last_error_msg());
		}
	}

	return $result;
}

function wpjam_send_json($response=[], $status_code=null){
	if(is_wp_error($response)){
		$errdata	= $response->get_error_data();
		$response	= [
			'errcode'	=> $response->get_error_code(), 
			'errmsg'	=> $response->get_error_message(),
		];

		if($errdata){
			$errdata	= is_array($errdata) ? $errdata : ['errdata'=>$errdata];
			$response 	= $response + $errdata;
		}
	}else{
		if(is_array($response)){
			if(!$response || !wp_is_numeric_array($response)){
				$response	= wp_parse_args($response, ['errcode'=>0]);
			}
		}elseif($response === true){
			$response	= ['errcode'=>0];
		}elseif($response === false || is_null($response)){
			$response	= ['errcode'=>'-1', 'errmsg'=>'系统数据错误或者回调函数返回错误'];
		}
	}

	$result	= wpjam_json_encode($response);

	if(!headers_sent() && !wpjam_doing_debug()){
		if(!is_null($status_code)){
			status_header($status_code);
		}

		if(wp_is_jsonp_request()){
			$result	= '/**/' . $_GET['_jsonp'] . '(' . $result . ')';

			$content_type	= 'application/jsjavascripton';
		}else{
			$content_type	= 'application/json';
		}

		@header('Content-Type: '.$content_type.'; charset='.get_option('blog_charset'));
	}

	echo $result;

	exit;
}

function wpjam_get_parameter($name, $args=[]){
	$object	= new WPJAM_Parameter($name, $args);

	return $object->get_value();
}

function wpjam_get_data_parameter($name='', $args=[]){
	if($name){
		$object	= new WPJAM_Parameter($name, $args);

		return $object->get_value('data');
	}else{
		return WPJAM_Parameter::get_data();
	}
}

function wpjam_get_defaults_parameter(){
	return WPJAM_Parameter::get_defaults();
}

function wpjam_method_allow($method, $send=true){
	if($_SERVER['REQUEST_METHOD'] != strtoupper($method)){
		$wp_error = new WP_Error('method_not_allow', '接口不支持 '.$_SERVER['REQUEST_METHOD'].' 方法，请使用 '.$method.' 方法！');

		return $send ? wpjam_send_json($wp_error): $wp_error;
	}

	return true;
}

function wpjam_http_request($url, $args=[], $err_args=[]){
	$object	= new WPJAM_Request($url, $args);

	return $object->request($err_args);
}

function wpjam_remote_request($url, $args=[], $err_args=[]){
	return wpjam_http_request($url, $args, $err_args);
}


function wpjam_get_setting($option, $name, $blog_id=0){
	return WPJAM_Setting::get_instance('option', $option, $blog_id)->get_setting($name);
}

function wpjam_update_setting($option, $name, $value, $blog_id=0){
	return WPJAM_Setting::get_instance('option', $option, $blog_id)->update_setting($name, $value);
}

function wpjam_delete_setting($option, $name, $blog_id=0){
	return WPJAM_Setting::get_instance('option', $option, $blog_id)->delete_setting($name);
}

function wpjam_get_option($name, $blog_id=0, $default=[], $type='option'){
	if($type == 'option'){
		$object	= WPJAM_Setting::get_instance('option', $name, $blog_id);
	}else{
		$object	= WPJAM_Setting::get_instance('site_option', $name);
	}

	if($default !== []){
		return $object->get_value($default);
	}

	return $object->get_option();
}

function wpjam_update_option($name, $value, $blog_id=0){
	return WPJAM_Setting::get_instance('option', $name, $blog_id)->update_option($value);
}

function wpjam_get_site_setting($option, $name){
	return WPJAM_Setting::get_instance('site_option', $option)->get_setting($name);
}

function wpjam_get_site_option($name, $default=[]){
	return wpjam_get_option($name, 0, $default, $type='site_option');
}

function wpjam_update_site_option($name, $value){
	return WPJAM_Setting::get_instance('site_option', $name)->update_option($value);
}

function wpjam_sanitize_option_value($value){
	return WPJAM_Setting::sanitize_option($value);
}


function wpjam_get_post_id_field($post_type='post', $args=[]){
	return wpjam_get_data_type_field('post_type', array_merge($args, ['post_type'=>$post_type]));
}

function wpjam_get_posts($post_ids, $args=[]){
	$posts = WPJAM_Post::get_by_ids($post_ids, $args);

	return $posts ? array_values($posts) : [];
}

function wpjam_validate_post($post_id, $post_type=''){
	return WPJAM_Post::validate($post_id, $post_type);
}

function wpjam_get_post($post, $args=[]){
	$object	= WPJAM_Post::get_instance($post);

	return $object ? $object->parse_for_json($args) : null;
}

function wpjam_get_post_views($post=null){
	$object	= WPJAM_Post::get_instance($post);

	return $object ? $object->views : 0;
}

function wpjam_update_post_views($post=null, $addon=1){
	$object	= WPJAM_Post::get_instance($post);

	return $object ? $object->view($addon) : null;
}

function wpjam_get_post_excerpt($post=null, $length=0, $more=null){
	$object	= WPJAM_Post::get_instance($post);

	return $object ? $object->get_excerpt($length, $more) : '';
}

function wpjam_get_post_content($post=null, $raw=false){
	$object	= WPJAM_Post::get_instance($post);

	return $object ? $object->get_content($raw) : '';
}

function wpjam_get_post_images($post=null, $large_size='large', $thumbnail_size='thumbnail'){
	$object	= WPJAM_Post::get_instance($post);

	return $object ? $object->get_images($large_size, $thumbnail_size) : [];
}

function wpjam_get_post_thumbnail_url($post=null, $size='full', $crop=1){
	$object	= WPJAM_Post::get_instance($post);

	return $object ? $object->get_thumbnail_url($size, $crop) : '';
}

function wpjam_get_post_first_image_url($post=null, $size='full'){
	$object	= WPJAM_Post::get_instance($post);

	return $object ? $object->get_first_image_url($size) : '';
}

// WP_Query 缓存
function wpjam_query($args=[]){
	return new WP_Query(wp_parse_args($args, [
		'no_found_rows'			=> true,
		'ignore_sticky_posts'	=> true,
		'cache_it'				=> true
	]));
}

function wpjam_parse_query_vars($query_vars){
	return WPJAM_Route::parse_query_vars($query_vars);
}

function wpjam_parse_query($wp_query, $args=[], $parse_for_json=true){
	$object	= new WPJAM_Query_Parser($wp_query, $args);

	return $parse_for_json ? $object->parse($args) : $object->render($args);
}

function wpjam_render_query($wp_query, $args=[]){
	return wpjam_parse_query($wp_query, $args, false);
}

function wpjam_related_posts($args=[]){
	echo wpjam_get_related_posts(null, $args, false);
}

function wpjam_get_related_posts($post=null, $args=[], $parse_for_json=false){
	$wp_query	= wpjam_get_related_posts_query($post, $args);

	if($parse_for_json){
		$args['filter']	= 'wpjam_related_post_json';
	}

	return wpjam_parse_query($wp_query, $args, $parse_for_json);
}

// wpjam_get_related_posts_query($number);
// wpjam_get_related_posts_query($post_id, $args);
function wpjam_get_related_posts_query(...$args){
	if(count($args) <= 1){
		$post_id	= get_the_ID();
		$args		= ['number'=>$args[0] ?? 5];
	}else{
		$post_id	= $args[0];
		$args		= $args[1];
	}

	$object	= WPJAM_Post::get_instance($post_id);

	return $object ? $object->get_related_query($args) : false;
}

function wpjam_get_new_posts($args=[], $parse_for_json=false){
	$query_vars	= [
		'post_type'			=> 'post',
		'post_status'		=> 'publish',
		'posts_per_page'	=> 5,
		'orderby'			=> 'date',
	];

	return wpjam_parse_query($query_vars, $args, $parse_for_json);
}

function wpjam_get_top_viewd_posts($args=[], $parse_for_json=false){
	$query_vars	= [
		'post_type'			=> 'post',
		'post_status'		=> 'publish',
		'posts_per_page'	=> 5,
		'orderby'			=> 'meta_value_num',
		'meta_key'			=> 'views',
	];

	return wpjam_parse_query($query_vars, $args, $parse_for_json);
}

function wpjam_get_permastruct($name){
	return $GLOBALS['wp_rewrite']->extra_permastructs[$name]['struct'] ?? '';
}

function wpjam_set_permastruct($name, $value){
	return $GLOBALS['wp_rewrite']->extra_permastructs[$name]['struct']	= $value;
}


function wpjam_get_related_object_ids($tt_ids, $number, $page=1){
	$id_str		= implode(',', array_map('intval', $tt_ids));
	$cache_key	= 'related_object_ids:'.$id_str.':'.$page.':'.$number;
	$object_ids	= wp_cache_get($cache_key, 'terms');

	if($object_ids === false){
		$object_ids	= $GLOBALS['wpdb']->get_col('SELECT object_id, count(object_id) as cnt FROM '.$GLOBALS['wpdb']->term_relationships.' WHERE term_taxonomy_id IN ('.$id_str.') GROUP BY object_id ORDER BY cnt DESC, object_id DESC LIMIT '.(($page-1) * $number).', '.$number);

		wp_cache_set($cache_key, $object_ids, 'terms', DAY_IN_SECONDS);
	}

	return $object_ids;
}

function wpjam_get_taxonomy_query_key($taxonomy){
	$query_keys	= ['category'=>'cat', 'post_tag'=>'tag_id'];

	return $query_keys[$taxonomy] ?? $taxonomy.'_id';
}

function wpjam_get_term_id_field($taxonomy='category', $args=[]){
	return wpjam_get_data_type_field('taxonomy', array_merge($args, ['taxonomy'=>$taxonomy]));
}

/**
* $max_depth = -1 means flatly display every element.
* $max_depth = 0 means display all levels.
* $max_depth > 0 specifies the number of display levels.
*/
function wpjam_get_terms($args, $max_depth=null){
	if(is_string($args) || wp_is_numeric_array($args)){
		$term_ids	= wp_parse_id_list($args);

		if(empty($term_ids)){
			return [];
		}

		$args		= ['orderby'=>'include', 'include'=>$term_ids];
		$max_depth	= $max_depth ?? -1;
	}else{
		$taxonomy	= $args['taxonomy'] ?? '';
		$tax_obj	= ($taxonomy && is_string($taxonomy)) ? get_taxonomy($taxonomy) : null;

		if(!$tax_obj){
			return [];
		}

		if(is_null($max_depth)){
			if($tax_obj->hierarchical){
				$max_depth	= wpjam_get_taxonomy_setting($taxonomy, 'levels', 0);
			}else{
				$max_depth	= -1;
			}
		}

		if($max_depth != -1){
			if(isset($args['child_of'])){
				$parent	= $args['child_of'];
			}else{
				$parent	= wpjam_array_pull($args, 'parent');

				if($parent){
					$args['child_of']	= $parent;
				}
			}
		}
	}

	$args	= wp_parse_args($args, ['hide_empty'=>false]);
	$terms	= get_terms($args) ?: [];

	if(is_wp_error($terms) || empty($terms)){
		return $terms;
	}

	$children	= [];

	if($max_depth != -1){
		$top_level	= [];

		if($parent){
			$top_level[] = get_term($parent);
		}

		foreach($terms as $term){
			if($term->parent == 0){
				$top_level[] = $term;
			}elseif($max_depth != 1){
				$children[$term->parent][] = $term;
			}
		}

		$terms	= $top_level;
	}

	foreach($terms as &$term){
		$term	= WPJAM_Term::get_instance($term)->parse_for_json($children, $max_depth, 0);
	}

	return $terms;
}

function wpjam_validate_term($term_id, $taxonomy=''){
	return WPJAM_Term::validate($term_id, $taxonomy);
}

function wpjam_get_term($term, $taxonomy=''){
	$term	= wpjam_validate_term($term, $taxonomy);

	if(is_wp_error($term)){
		return $term;
	}

	$object	= WPJAM_Term::get_instance($term);

	return $object->parse_for_json();
}

function wpjam_get_term_thumbnail_url($term=null, $size='full', $crop=1){
	$object	= WPJAM_Term::get_instance($term);

	return $object ? $object->get_thumbnail_url($size, $crop) : '';
}

function wpjam_get_term_level($term){
	$object	= WPJAM_Term::get_instance($term);

	return $object ? $object->get_level() : '';
}


function wpjam_get_user($user, $size=96){
	$object	= WPJAM_User::get_instance($user);

	return $object ? $object->parse_for_json($size) : null;
}

function wpjam_get_authors($args=[], $return='users'){
	if(version_compare($GLOBALS['wp_version'], '5.9', '<')){
		$args['who']		= 'authors';
	}else{
		$args['capability']	= ['edit_posts'];
	}

	return $return == 'args' ? $args : get_users($args);
}


function wpjam_add_admin_notice($notice, $blog_id=0){
	if(is_multisite()){
		$blog_id	= $blog_id ?: get_current_blog_id();

		if(!get_site($blog_id)){
			return;
		}
	}else{
		$blog_id	= get_current_blog_id();
	}

	return WPJAM_Notice::get_instance('admin_notice', $blog_id)->insert($notice);
}

function wpjam_add_user_notice($user_id, $notice){
	if(get_userdata($user_id)){
		return WPJAM_Notice::get_instance('user_notice', $user_id)->insert($notice);
	}
}

function wpjam_fields($fields, $args=[]){
	return (WPJAM_Fields::create($fields))->render($args);
}

function wpjam_validate_fields_value($fields, $values=null){
	if(is_wp_error($fields)){
		return $fields;
	}

	return (WPJAM_Fields::create($fields))->validate($values);
}

function wpjam_prepare_fields_value($fields, $args=[]){
	return (WPJAM_Fields::create($fields))->prepare($args);
}

function wpjam_get_fields_defaults($fields){
	return (WPJAM_Fields::create($fields))->get_defaults();
}

function wpjam_field($field, $args=[]){
	return (WPJAM_Field::create($field))->render($args);
}

function wpjam_validate_field_value($field, $value){
	return (WPJAM_Field::create($field))->validate($value);
}

function wpjam_field_get_icon($name){
	return WPJAM_Field::get_icon($name);
}


function wpjam_get_filter_name($name='', $type=''){
	$filter	= str_replace('-', '_', $name);
	$filter	= str_replace('wpjam_', '', $filter);

	return 'wpjam_'.$filter.'_'.$type;
}

function wpjam_is_login(){
	if(preg_match('#(wp-login\.php)([?/].*?)?$#i', $_SERVER['PHP_SELF'])){
		return true;
	}

	return false;
}

function wpjam_get_filesystem(){
	if(empty($GLOBALS['wp_filesystem'])){
		if(!function_exists('WP_Filesystem')){
			require_once(ABSPATH.'wp-admin/includes/file.php');
		}

		WP_Filesystem();
	}

	return $GLOBALS['wp_filesystem'];
}

