<?php
class WPJAM_Grants_Admin{
	public static function render_item($appid='', $secret=''){
		if($appid){
			$secret	= $secret ? '<p class="secret" id="secret_'.$appid.'" style="display:block;">'.$secret.'</p>' : '<p class="secret" id="secret_'.$appid.'"></p>';
			$times	= WPJAM_Grant::get_instance()->cache_get_item($appid, 'token.grant') ?: 0;
			$data	= ['data'=>['appid'=>$appid]];

			return '
			<tbody id="appid_'.$appid.'">
				<tr>
					<th>AppID</th>
					<td class="appid">'.$appid.'</td>
					<td>'.wpjam_get_page_button('delete_grant', $data).'</td>
				</tr>
				<tr>
					<th>Secret</th>
					<td>出于安全考虑，Secret不被明文保存，忘记密钥请点击重置：'.$secret.'</td>
					<td>'.wpjam_get_page_button('reset_secret', $data).'</td>
				</tr>
				<tr>
					<th>用量</th>
					<td>鉴权接口已被调用了 <strong>'.$times.'</strong> 次，更多接口调用统计请点击'.wpjam_get_page_button('get_stats', $data).'</td>
					<td>'.wpjam_get_page_button('clear_quota', $data).'</td>
				</tr>
			</tbody>
			';
		}else{
			return '
			<tbody id="create_grant">
				<tr>
					<th>创建</th>
					<td>点击右侧创建按钮可创建 AppID/Secret，最多可创建三个：</td>
					<td>'.wpjam_get_page_button('create_grant').'</td>
				</tr>
			</tbody>
			';
		}
	}

	public static function get_fields($name){
		if($name == 'get_stats'){
			$appid	= wpjam_get_data_parameter('appid');
			$caches	= WPJAM_Grant::get_instance()->cache_get($appid);
			$fields	= [];

			if($appid){
				$fields['appid']	= ['title'=>'APPID',	'type'=>'view', 'value'=>$appid];

				$caches['token.grant']	= $caches['token.grant'] ?? 0;
			}

			if($caches){
				foreach($caches as $json => $times){
					$fields[$json]	= ['title'=>$json,	'type'=>'view', 'value'=>$times];
				}
			}else{
				$fields['no']	= ['type'=>'view', 'value'=>'暂无数据'];
			}

			return $fields;
		}elseif($name == 'get_doc'){
			$doc	= '
			<p>access_token 是开放接口的全局<strong>接口调用凭据</strong>，第三方调用各接口时都需使用 access_token，开发者需要进行妥善保存。</p>
			<p>access_token 的有效期目前为2个小时，需定时刷新，重复获取将导致上次获取的 access_token 失效。</p>

			<h4>请求地址</h4>

			<p><code>'.home_url('/api/').'token/grant.json?appid=APPID&secret=APPSECRET</code></p>

			<h4>参数说明<h4>

			'.do_shortcode('[table th=1 class="form-table striped"]
			参数	
			是否必须
			说明

			appid
			是
			第三方用户凭证

			secret
			是
			第三方用户证密钥。
			[/table]').'
			
			<h4>返回说明</h4>

			<p><code>
				{"errcode":0,"access_token":"ACCESS_TOKEN","expires_in":7200}
			</code></p>';

			return ['access_token'=>['type'=>'view', 'value'=>$doc]];
		}			
	}

	public static function ajax_response($name){
		$object	= WPJAM_Grant::get_instance();

		if($name == 'create_grant'){
			$appid	= $object->create();

			if(is_wp_error($appid)){
				wpjam_send_json($appid);
			}

			$secret	= $object->reset_secret($appid);

			wpjam_send_json(['item'=>self::render_item($appid, $secret)]);
		}else{
			$appid	= wpjam_get_data_parameter('appid');

			if($name == 'delete_grant'){
				$result	= $object->delete($appid);

				if(is_wp_error($result)){
					wpjam_send_json($result);
				}else{
					wpjam_send_json(compact('appid'));
				}
			}elseif($name == 'reset_secret'){
				$secret	= $object->reset_secret($appid);

				if(is_wp_error($secret)){
					wpjam_send_json($secret);
				}else{
					wpjam_send_json(compact('appid', 'secret'));
				}
			}elseif($name == 'clear_quota'){
				$object->cache_delete($appid);

				wpjam_send_json(['errmsg'=>'接口已清零']);
			}
		}
	}

	public static function load_plugin_page(){
		wpjam_register_page_action('create_grant', [
			'button_text'	=> '创建',
			'class'			=> 'button',
			'direct'		=> true,
			'confirm'		=> true,
			'callback'		=> [self::class, 'ajax_response']
		]);

		wpjam_register_page_action('reset_secret', [
			'button_text'	=> '重置',
			'class'			=> 'button',
			'direct'		=> true,
			'confirm'		=> true,
			'callback'		=> [self::class, 'ajax_response']
		]);

		wpjam_register_page_action('delete_grant', [
			'button_text'	=> '删除',
			'class'			=> 'button',
			'direct'		=> true,
			'confirm'		=> true,
			'callback'		=> [self::class, 'ajax_response']
		]);

		wpjam_register_page_action('clear_quota', [
			'button_text'	=> '清零',
			'class'			=> 'button button-primary',
			'direct'		=> true,
			'confirm'		=> true,
			'callback'		=> [self::class, 'ajax_response']
		]);

		wpjam_register_page_action('get_stats', [
			'button_text'	=> '用量',
			'submit_text'	=> '',
			'class'			=> '',
			'width'			=> 500,
			'fields'		=> [self::class, 'get_fields']
		]);

		wpjam_register_page_action('get_doc', [
			'button_text'	=> '接口文档',
			'submit_text'	=> '',
			'page_title'	=> '获取access_token',
			'class'			=> 'page-title-action button',
			'fields'		=> [self::class, 'get_fields']
		]);
	}

	public static function plugin_page(){
		echo '<div class="card">';

		echo '<h3>开发者 ID<span class="page-actions">'.wpjam_get_page_button('get_doc').'</span></h3>';

		echo '<table class="form-table widefat striped">';

		foreach(WPJAM_Grant::get_instance()->get_items() as $item){
			echo self::render_item($item['appid']); 
		}

		echo self::render_item();

		echo '</table>';
		
		echo '</div>';
	}
}

add_action('admin_head', function(){ ?>
	<style type="text/css">
	div.card{max-width:640px;}
	
	div.card table.form-table{border: none;}
	div.card table.form-table tbody:after{content: ' '; margin-bottom: 40px; display: block;}
	div.card table.form-table tbody:nth-of-type(4){display: none;}
	
	table.form-table th{width: 60px; padding-left: 10px;}

	table.form-table td.appid{font-weight: bold;}
	table.form-table p.secret{display: none; background: #ffc; padding:4px 8px; font-weight: bold;}

	div.card h3{margin-bottom: 40px;}
	div.card h3 span.page-actions{float:right;}
	</style>

	<script type="text/javascript">
	jQuery(function($){
		$('body').on('page_action_success', function(e, response){
			if(response.page_action == 'create_grant'){
				$('#create_grant').before(response.item);
			}else if(response.page_action == 'delete_grant'){
				$('#appid_'+response.appid).remove();
			}else if(response.page_action == 'reset_secret'){
				$('#secret_'+response.appid).show().html(response.secret);
			}
		});
	});
	</script>
<?php });